/*
 * Copyright (c) 2002-2003 by OpenSymphony
 * All rights reserved.
 */
package com.opensymphony.user.provider.file;

import com.opensymphony.user.Entity;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;


/**
 * @author Dave Brondsema
 */
abstract public class FileAccessProvider implements com.opensymphony.user.provider.AccessProvider {
    //~ Static fields/initializers /////////////////////////////////////////////

    protected static final Log log = LogFactory.getLog(FileAccessProvider.class);

    //~ Instance fields ////////////////////////////////////////////////////////

    protected FileGroupsCache groupCache; /* this must be set by any child class's init() function */

    //~ Methods ////////////////////////////////////////////////////////////////

    public boolean addToGroup(String username, String groupname) {
        if (!inGroup(username, groupname)) {
            boolean rv = getGroup(groupname).users.add(username);

            return rv && groupCache.store();
        }

        return false;
    }

    public boolean create(String name) {
        if (groupCache.groups.containsKey(name)) {
            return false;
        }

        FileGroup group = new FileGroup();
        group.name = name;
        groupCache.groups.put(name, group);

        return groupCache.store();
    }

    public void flushCaches() {
        groupCache.store();
    }

    public boolean handles(String name) {
		if (groupCache == null) {
			return false;
		}
		return groupCache.groups.containsKey(name);
		// @TODO: Major hack
		/*
        if (!handle) {
            handle = new SerializableCredentialsProvider().handles(name);
        }

        return handle;
        */

        /*

        this needs to check if this access provider handles groups AND users

        but it has no reference to the in memory users


        old version:

        return groups.containsKey( name );

        */
    }

    public boolean inGroup(String username, String groupname) {
        FileGroup group = getGroup(groupname);

        return (group != null) && group.users.contains(username);
    }

    public boolean init(Properties properties) {
        return true;
    }

    public List list() {
        return Collections.unmodifiableList(new ArrayList(groupCache.groups.keySet()));
    }

    public List listGroupsContainingUser(String username) {
        List result = new ArrayList();
        Iterator i = groupCache.groups.keySet().iterator();

        while (i.hasNext()) {
            String currentGroup = (String) i.next();

            if (inGroup(username, currentGroup)) {
                result.add(currentGroup);
            }
        }

        return Collections.unmodifiableList(result);
    }

    public List listUsersInGroup(String groupname) {
        FileGroup g = getGroup(groupname);

        if (g == null) {
            return Collections.EMPTY_LIST;
        }

        return Collections.unmodifiableList(getGroup(groupname).users);
    }

    public boolean load(String name, Entity.Accessor accessor) {
        accessor.setMutable(true);

        return true;
    }

    public boolean remove(String name) {
        boolean rv = groupCache.groups.remove(name) != null;

        return rv && groupCache.store();
    }

    public boolean removeFromGroup(String username, String groupname) {
        boolean rv = getGroup(groupname).users.remove(username);

        return rv && groupCache.store();
    }

    public boolean store(String name, Entity.Accessor accessor) {
        return groupCache.store();
    }

    private FileGroup getGroup(String groupname) {
        FileGroup group = (FileGroup) groupCache.groups.get(groupname);

        return group;
    }
}
