/*
 * Copyright (c) 2002-2003 by OpenSymphony
 * All rights reserved.
 */
package com.opensymphony.user.provider.file;

import com.opensymphony.user.Entity;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;


/**
* Following properties are <b>required</b>:
* <ul>
* <li><b>storeFile</b> - the absolute path to the store file (<i>ex:c:\\users.store</i>)</li>
* </ul>
*/
public class FileCredentialsProvider implements com.opensymphony.user.provider.CredentialsProvider {
    //~ Static fields/initializers /////////////////////////////////////////////

    private static final Log log = LogFactory.getLog(FileCredentialsProvider.class);

    //~ Instance fields ////////////////////////////////////////////////////////

    protected FileUsersCache userCache;

    //~ Methods ////////////////////////////////////////////////////////////////

    public boolean authenticate(String name, String password) {
        FileUser user = (FileUser) userCache.users.get(name);

        if (user == null) {
            return false;
        }

        return password.equals(user.password);
    }

    public boolean changePassword(String name, String password) {
        FileUser user = (FileUser) userCache.users.get(name);

        if (user == null) {
            return false;
        }

        user.password = password;

        return userCache.store();
    }

    public boolean create(String name) {
        if (userCache.users.containsKey(name)) {
            return false;
        }

        FileUser user = new FileUser();
        user.name = name;
        userCache.users.put(name, user);

        return userCache.store();
    }

    public void flushCaches() {
		userCache.store();
    }

    public boolean handles(String name) {
        if (userCache == null) {
            return false;
        }

        return userCache.users.containsKey(name);
    }

    public boolean init(Properties properties) {
        return true;
    }

    public List list() {
        return Collections.unmodifiableList(new ArrayList(userCache.users.keySet()));
    }

    public boolean load(String name, Entity.Accessor accessor) {
        accessor.setMutable(true);

        return true;
    }

    public boolean remove(String name) {
        boolean rv = userCache.users.remove(name) != null;

        return rv && userCache.store();
    }

    public boolean store(String name, Entity.Accessor accessor) {
        return userCache.store();
    }
}
