/*
 * Copyright (c) 2002-2003 by OpenSymphony
 * All rights reserved.
 */
package com.opensymphony.user.provider.file;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * @author Dave Brondsema
 */
abstract class FileEntitiesCache {
    //~ Static fields/initializers /////////////////////////////////////////////

    protected static final Log log = LogFactory.getLog(FileEntitiesCache.class);

    //~ Instance fields ////////////////////////////////////////////////////////

    protected String storeFile;
    protected String storeFileType;

    //~ Constructors ///////////////////////////////////////////////////////////

    public FileEntitiesCache(String storeFile, String storeFileType) {
        this.storeFile = storeFile;
        this.storeFileType = storeFileType;

        if (storeFile == null) {
            log.fatal("property storeFile must be specified");
        }

        if (storeFileType == null) {
            log.fatal("property storeFileType must be specified; one of (file,resource)");
        }

        if (!storeFileType.equalsIgnoreCase("file") && !storeFileType.equalsIgnoreCase("resource")) {
            log.fatal("property storeFileType must be one of (file,resource)");
        }

    }

    //~ Methods ////////////////////////////////////////////////////////////////

    abstract public boolean load();

    abstract public boolean store();

    protected InputStream getInputStreamFromStoreFile() throws IOException {
        if (storeFileType.equalsIgnoreCase("file")) {
            return new FileInputStream(storeFile);
        } else if (storeFileType.equalsIgnoreCase("resource")) {
            ClassLoader classLoader = Thread.currentThread().getContextClassLoader();

            return classLoader.getResourceAsStream(storeFile);
        } else {
            log.fatal("property storeFileType must be one of (file,resource)");

            return null;
        }
    }
}
