/*
 * Copyright (c) 2002-2003 by OpenSymphony
 * All rights reserved.
 */
package com.opensymphony.user.provider.file;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;


class SerializableUsersCache extends FileUsersCache implements Serializable {
    //~ Static fields/initializers /////////////////////////////////////////////

    private static final Log log = LogFactory.getLog(SerializableUsersCache.class);

    //~ Constructors ///////////////////////////////////////////////////////////

    public SerializableUsersCache(String storeFile, String storeFileType) {
        super(storeFile, storeFileType);
        load();
    }

    //~ Methods ////////////////////////////////////////////////////////////////

    public boolean load() {
        try {
            ObjectInputStream ois = new ObjectInputStream(getInputStreamFromStoreFile());
            SerializableUsersCache u = (SerializableUsersCache) ois.readObject();

            users = u.users;

            return true;
        } catch (Exception e) {
            log.fatal("cannot load from file " + storeFile + ". Create a new blank store.", e);
        }

        return false;
    }

    public boolean store() {
        try {
            FileOutputStream fos = new FileOutputStream(new File(storeFile));
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeObject(this);
            fos.close();

            return true;
        } catch (Exception e) {
            log.fatal("cannot store in file " + storeFile + ".", e);
        }

        return false;
    }
}
