/*
 * Copyright (c) 2002-2003 by OpenSymphony
 * All rights reserved.
 */
package com.opensymphony.user.provider.file;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Properties;


/**
* Following properties are <b>required</b>:
* <ul>
* <li><b>storeFile</b> - the absolute path to the store file (<i>ex: c:\groups.xml</i>) or a resource location in the jar (<i>ex: /groups.xml</i>) </li>
* <li><b>storeFileType</b> - one of (file, resource)
* </ul>
*
* XML format example:
* <pre>
* &lt;groups&gt;
*   &lt;group id="myGroup"&gt;
*     &lt;user id="myUser"/&gt;
*     &lt;user id="someOtherUser"/&gt;
*   &lt;/group&gt;
*   &lt;group id="anEmptyGroup"/&gt;
*   &lt;group id="foo"&gt;
*     &lt;user id="bar"/&gt;
*     &lt;user id="baz"/&gt;
*   &lt;/group&gt;
* &lt;/groups&gt;
* </pre>
*
* @author Dave Brondsema
*
*/
public class XMLAccessProvider extends FileAccessProvider {
    //~ Static fields/initializers /////////////////////////////////////////////

    protected static final Log log = LogFactory.getLog(SerializableAccessProvider.class);

    //~ Methods ////////////////////////////////////////////////////////////////

    public boolean init(Properties properties) {
        boolean retVal = super.init(properties);

        if (retVal) {
            groupCache = new XMLGroupsCache(properties.getProperty("storeFile"), properties.getProperty("storeFileType"));

            return true;
        }

        return false;
    }
}
