/*
 * Copyright (c) 2002-2003 by OpenSymphony
 * All rights reserved.
 */
package com.opensymphony.user.provider.file;

import java.util.Properties;


/**
* Following properties are <b>required</b>:
* <ul>
* <li><b>storeFile</b> - the absolute path to the store file (<i>ex: c:\groups.xml</i>) or a resource location in the jar (<i>ex: /groups.xml</i>) </li>
* <li><b>storeFileType</b> - one of (file, resource)
* </ul>
*
* XML format example:
* <pre>
* &lt;users&gt;
*   &lt;user id="jdoe"&gt;
*     &lt;property-set&gt;
*       &lt;property key="fullName" type="String"&gt;John Doe&lt;property/&gt;
*       &lt;property key="email" type="String"&gt;john.doe@myplace.net&lt;property/&gt;
*     &lt;/property-set&gt;
*   &lt;/user&gt;
*   &lt;user id="myUser"&gt;
*     &lt;property-set&gt;
*       &lt;property key="abc" type="String"&gt;def&lt;property/&gt;
*     &lt;/property-set&gt;
*   &lt;/user&gt;
*   &lt;user id="foo"/&gt;
* &lt;/users&gt;
* </pre>
*
* @author Dave Brondsema
*
*/
public class XMLProfileProvider extends FileProfileProvider {
    //~ Methods ////////////////////////////////////////////////////////////////

	public boolean init(Properties properties) {
		boolean retVal = super.init(properties);

		if (retVal) {
			propertySetCache = new XMLPropertySetCache(properties.getProperty("storeFile"), properties.getProperty("storeFileType"));

			return true;
		}

		return false;
	}
}
