/*
 * Copyright (c) 2002-2003 by OpenSymphony
 * All rights reserved.
 */
package com.opensymphony.user.provider.file;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.w3c.dom.*;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.module.propertyset.PropertySetManager;
import com.opensymphony.module.propertyset.xml.XMLPropertySet;

import java.io.FileWriter;
import java.io.StringReader;

import java.util.HashMap;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilderFactory;


class XMLPropertySetCache extends FilePropertySetCache {
    //~ Static fields/initializers /////////////////////////////////////////////

    protected static final Log log = LogFactory.getLog(XMLPropertySetCache.class);

    //~ Constructors ///////////////////////////////////////////////////////////

    public XMLPropertySetCache(String storeFile, String storeFileType) {
        super(storeFile, storeFileType);
        load();
    }

    //~ Methods ////////////////////////////////////////////////////////////////

    public boolean load() {
        try {
            // clear the groups first
            propertySets = new HashMap();

			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			
            Document document = factory.newDocumentBuilder().parse(getInputStreamFromStoreFile());
            NodeList users = document.getDocumentElement().getChildNodes();

            if (log.isDebugEnabled()) {
                log.debug("loaded " + storeFile);
            }

            // read each <user>
            for (int i = 0; i < users.getLength(); i++) {
                Element userElement = (Element) users.item(i);

                if (userElement.getNodeName().equals("user")) {
                    if (userElement.getAttribute("id") == null) {
                        log.warn("attribute 'id' required for <user/> in " + storeFile);

                        continue;
                    }

                    String userName = userElement.getAttribute("id");
                    Node propertySetContent = userElement.getFirstChild();
                    log.debug(propertySetContent.toString());
                    
					XMLPropertySet ps = (XMLPropertySet)PropertySetManager.getInstance("xml", null);
					Document d = factory.newDocumentBuilder().newDocument();
					d.appendChild(propertySetContent);
					ps.loadFromDocument(d);
					propertySets.put(userName, ps);
					
                    log.debug("added user credentials " + userName);
                } else {
                    log.warn("expected 'user' element, found " + userElement.getNodeName() + " in " + storeFile);
                }
            }

            return true;
        } catch (Exception e) {
            log.fatal("cannot load from file " + storeFile + ".", e);
        }

        return false;
    }

    public boolean store() {
        try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            Document document = factory.newDocumentBuilder().newDocument();
            Node root = document.createElement("users");

            // a <user id="foo" password="bar"> for each group
            for (Iterator i = propertySets.keySet().iterator(); i.hasNext();) {
                String userKey = i.next().toString();
                XMLPropertySet ps = (XMLPropertySet) propertySets.get(userKey);
                //Node user = createElement("user").addAttribute("id", userKey);
                Node userNode = new Node();
                root.appendChild(userNode);
                log.debug(ps.saveToDocument().toString());
                //userElement.add(ps.saveToDocument().toString());
            }

            // print it nicely
            OutputFormat format = OutputFormat.createPrettyPrint();
            XMLWriter writer = new XMLWriter(new FileWriter(storeFile));
            writer.write(document);
            writer.close();

            return true;
        } catch (Exception e) {
            log.fatal("cannot store in file " + storeFile + ".", e);
        }

        return false;
    }
}
